import pandas as pd
from . import datafile

class NodeHealthFile(datafile.DataFile):
    """Node Health File"""
    def __init__(self, filename):
        super(NodeHealthFile, self).__init__(filename)

    def _clean(self):
        pass


import logging
import utm
import pandas as pd

from . import dataset

class LocationApi:
    """utility for localizing tags in merged beep/node dataset"""
    def __init__(self, beep_filename, node_filename):
        """filenames of raw beep data and node location data"""
        self.beeps = dataset.BeepLocationDataset(beep_filename, node_filename) 
        # ez reference
        self.merged_df = self.beeps.df

        # assume utm zone, letter from first record
        self.zone = self.merged_df.iloc[0].zone
        self.letter = self.merged_df.iloc[0].letter

    def weighted_average(self, freq):
        """generate weighted average location dataset for given channel, tag_id and frequency"""
        mean_df = self.merged_df.resample(freq).mean()

        lats = []
        lngs = []
        for i, record in mean_df.iterrows():
            lat,lng = utm.to_latlon(record.node_x, record.node_y, self.zone, self.letter)
            lats.append(lat)
            lngs.append(lng)

        out_df = pd.DataFrame({
            'lat': lats,
            'lng': lngs,
        }, index=mean_df.index)

        # mean x,y from weighted node_x, node_y dataframe
        out_df['easting'] = mean_df.node_x
        out_df['northing'] = mean_df.node_y
        return out_df




import logging
import pandas as pd
import utm
from . import datafile

class RawBeepFile(datafile.DataFile):
    """sensor station beep file"""
    FILTER_MAX_RSSI = -20

    def __init__(self, filename):
        super(RawBeepFile, self).__init__(filename)

    def _clean(self):
        """convert Time column to datetime

        drop na records
        consistent node id"""
        df = self.df
        df = pd.read_csv(self.filename)
        df.Time = pd.to_datetime(df.Time)

        # drop na records
        pre_count = df.shape[0]
        df = df.dropna()
        dropped_count = pre_count - df.shape[0]
        if dropped_count > 0:
            logging.info('dropped {:,} n/a records from {:,} records'.format(dropped_count, pre_count))
        df = df.set_index('Time')

        # drop abnormal RSSI values
        df = df[df.TagRSSI < self.FILTER_MAX_RSSI]
        return df

    def beep_count(self):
        return self.df.shape[0]



import os
import datetime
from pytz import utc
import glob
import pandas as pd
import logging
logging.basicConfig(format='%(asctime)s: %(message)s', level=logging.INFO)

class DataManager:
    """Utility for managing CTT Sensor Station data files"""
    beep_pattern = '*-data*'
    gps_pattern = '*-gps*'
    health_pattern = '*-node*'

    def __init__(self):
        """initialize data frames for beep data, node health data, and gps data"""
        self.beep_data = None
        self.health_data = None
        self.gps_data = None

    def load_data(self, directory_name):
        """load data files from a directory that contains all the compressed (or uncompressed) data files straight off the station"""
        # load beep data
        path = '{}{}{}'.format(directory_name, os.path.sep, self.beep_pattern)
        beep_files = glob.glob(path)
        dfs_to_merge = []
        logging.info('preparing {:,} beep files for merge from {}'.format(len(beep_files), path))
        for filename in sorted(beep_files):
            if filename.find('node') == -1:
                # only process if node is not part of the pattern ...
                logging.info('merging file: {}'.format(filename))
                try:
                    df = pd.read_csv(filename)
                except pd.errors.EmptyDataError:
                    logging.info('ignoring file {} - no data'.format(filename))
                df['Time'] = pd.to_datetime(df['Time'])
                dfs_to_merge.append(df)
        if len(dfs_to_merge) > 0:
            self.beep_data = pd.concat(dfs_to_merge, axis=0).sort_values('Time')
        else:
            logging.info('no beep files found in directory')
        
        path = '{}{}{}'.format(directory_name, os.path.sep, self.health_pattern)
        node_health_files = glob.glob(path)
        dfs_to_merge = []
        logging.info('preparing {:,} node health files for merge'.format(len(node_health_files)))
        for filename in sorted(node_health_files):
            logging.info('merging file: {}'.format(filename))
            try:
                df = pd.read_csv(filename)
            except pd.errors.EmptyDataError:
                logging.info('ignoring file {} - no data'.format(filename))

            df['Time'] = pd.to_datetime(df['Time'])
            dfs_to_merge.append(df)
        if len(dfs_to_merge) > 0:
            self.health_data = pd.concat(dfs_to_merge, axis=0).sort_values('Time')
        else:
            logging.info('no node health files found in directory')


        path = '{}{}{}'.format(directory_name, os.path.sep, self.gps_pattern)
        gps_files = glob.glob(path)
        dfs_to_merge = []
        logging.info('preparing {:,} gps files for merge'.format(len(gps_files)))
        for filename in sorted(gps_files):
            logging.info('merging file: {}'.format(filename))
            try:
                df = pd.read_csv(filename)
            except pd.errors.EmptyDataError:
                logging.info('ignoring file {} - no data'.format(filename))

            df['recorded at'] = pd.to_datetime(df['recorded at'])
            dfs_to_merge.append(df)
        if len(dfs_to_merge) > 0:
            self.gps_data = pd.concat(dfs_to_merge, axis=0).sort_values('recorded at')
        else:
            logging.info('no gps files found in directory')
    
    def export_data(self, begin=None, end=None, tags=None):
        """export loaded data into output merged files stamped with current time the script is run
            specify begin and end dates to only export data between a set of dates
        """
        now = datetime.datetime.utcnow()
        beeps = self.beep_data
        health = self.health_data
        gps = self.gps_data
        if begin is not None:
            if beeps is not None:
                beeps = beeps[beeps.Time > begin]
            if health is not None:
                health = health[health.Time > begin]
            if gps is not None:
                gps = gps[gps['recorded at'] > begin]
        if end is not None:
            if beeps is not None:
                beeps = beeps[beeps.Time < end]
            if health is not None:
                health = health[health.Time < end]
            if gps is not None:
                gps = gps[gps['recorded at'] < end]

        if beeps is not None:
            outfilename = 'station-beep-data.merged.{}.csv'.format(now.strftime('%Y-%m-%d_%H%M%S'))
            beeps.to_csv(outfilename, index=False)
        if health is not None:
            outfilename = 'station-health-data.merged.{}.csv'.format(now.strftime('%Y-%m-%d_%H%M%S'))
            health.to_csv(outfilename, index=False)
        if gps is not None:
            outfilename = 'station-gps-data.merged.{}.csv'.format(now.strftime('%Y-%m-%d_%H%M%S'))
            gps.to_csv(outfilename, index=False)


if __name__ == '__main__':
    import sys
    if len(sys.argv) < 2:
        print('expected a command line argument to specify the directory')
        sys.exit(-1)
    directory_name = sys.argv[1]

    manager = DataManager()
    manager.load_data(directory_name)

    # you can export data during a time window by specifying begin and end dates during export
    begin = datetime.datetime(2019,11,1).replace(tzinfo=utc)
    end = datetime.datetime(2019,11,7).replace(tzinfo=utc)
    #manager.export_data(begin=begin, end=end)
    manager.export_data()


import pandas as pd

class DataFile:
    """generic csv data file abstraction"""
    def __init__(self, filename):
        """load filename into a pandas dataframe df"""
        self.filename = filename
        df = pd.read_csv(filename)
        self.df = self._clean_node_id(df)
        self.df = self._clean()

    def _clean(self):
        """logic for preparing the data frame for a given file type"""
        pass

    def _clean_node_id(self, df):
        """strip leading 0 from NodeId"""
        print('cleaning node id from file {}'.format(self.filename))
        if 'NodeId' in df.columns:
            print('found NodeId column')
            df.NodeId = df.NodeId.apply(lambda record: record.lstrip("0").upper())
        return df


import setuptools
with open("README.MD", "r") as inFile:
    long_description = inFile.read()

setuptools.setup(
    name="station-tools",
    version="0.0.31",
    author="Bob Fogg",
    author_email="bob.fogg@celltracktech.com",
    description="A package for managing and analyzing CTT LifeTag datasets",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bobfogg/sensor-station-tools",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import pandas as pd
import utm
from . import datafile

class NodeLocationFile(datafile.DataFile):
    """Node Location File Encapsulation
    expected fields: NodeId, lat, lng"""
    def __init__(self, filename):
        super(NodeLocationFile, self).__init__(filename)

    def _clean(self):
        """rename lat,lng to node_lat, node_lng; append utm"""
        self.df = self.df.rename(columns={'lat': 'node_lat', 'lng': 'node_lng'})
        self.df = self.append_utm(self.df)
        return self.df

    def append_utm(self, df):
        """append utm x,y,zone,letter to dataframe"""
        xs = []
        ys = []
        zones = []
        letters = []
        for i, record in df.iterrows():
            (x,y,zone,letter) = utm.from_latlon(record.node_lat, record.node_lng)
            xs.append(x)
            ys.append(y)
            zones.append(zone)
            letters.append(letter)
        df['node_x'] = xs
        df['node_y'] = ys
        df['zone'] = zones
        df['letter'] = letters
        return df

if __name__ == '__main__':
    import sys
    filename = sys.argv[1]
    data = NodeLocationFile(filename)
    print(data.df)


import logging
import pandas as pd
from ..data.rawbeepfile import RawBeepFile
from ..data.nodelocationfile import NodeLocationFile

logging.basicConfig(level=logging.INFO)
class BeepLocationDataset:
    """dataframe for merged beep / node location"""
    def __init__(self, beep_filename, node_filename):
        """filenames of raw beep data and node location data"""
        logging.info('loading datasets from beep file:{};  node location file:{}'.format(beep_filename, node_filename))
        self.beeps = RawBeepFile(filename=beep_filename)
        self.nodes = NodeLocationFile(filename=node_filename)
        # merged location dataframe
        self.df = None
        self._merge()

    def _merge(self):
        """merge beep dataset with node location dataset"""
        df = self.beeps.df.reset_index()
        print(self.nodes.df)
        print(df.NodeId.unique())
        df = df.merge(right=self.nodes.df, left_on='NodeId', right_on='NodeId')
        df = df.set_index('Time')
        delta = self.beeps.beep_count() - df.shape[0]
        if delta > 0:
            logging.error('dropped {:,} of {:,} records after merging node locations'.format(delta, self.beeps.beep_count()))
        self.df = df